import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import '../../../config/app_colors.dart';
import '../../../models/counts_by_hour.dart';
import '../../../utils/color_extensions.dart';

class CountsByHourCardContentChart extends StatelessWidget {
  final CountsByHourData chartData;

  const CountsByHourCardContentChart({super.key, required this.chartData});

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 120.0,
      child: LayoutBuilder(builder: (context, constraints) {
        final double maxWidth = constraints.maxWidth;
        final int barsCount = this.chartData.data.keys.length;
        final double barWidth = 0.7 * (maxWidth / barsCount);

        return BarChart(
          BarChartData(
            barGroups: getDataCounts(barWidth),
            backgroundColor: Theme.of(context).colorScheme.onSurface,
            borderData: getBorderData(),
            gridData: getGridData(),
            titlesData: getTitlesData(),
            barTouchData: getBarTouchData(),
            maxY: getNextRoundNumber(getMaxCountsValue(), 5),
          ),
        );
      }),
    );
  }

  double getMaxCountsValue() {
    double maxValue = 0;

    this.chartData.data.keys.forEach((key) {
      double? counts = this.chartData.data[key];
      if (counts != null) {
        if (counts > maxValue) {
          maxValue = counts;
        }
      }
    });

    return maxValue;
  }

  double getNextRoundNumber(double number, int scale) {
    return scale * ((number ~/ scale).toInt() + 1);
  }

  LinearGradient getGradient(Color baseColor) {
    return LinearGradient(
      begin: Alignment.topLeft,
      end: Alignment(0.8, 1),
      colors: <Color>[
        baseColor.lighten(30),
        baseColor,
        baseColor.darken(30),
      ],
      tileMode: TileMode.mirror,
    );
  }

  List<BarChartGroupData> getDataCounts(double barWidth) {
    List<BarChartGroupData> data = [];

    final baseColor = AppColors.contentColorCyan.darken(30);
    final gradient = getGradient(baseColor);
    final borderColor = baseColor.darken(20);

    this.chartData.data.keys.forEach((day) {
      final double? counts = this.chartData.data[day];

      if (counts != null) {
        data.add(BarChartGroupData(
          x: day,
          barRods: [
            BarChartRodData(
              toY: counts,
              color: baseColor,
              gradient: gradient,
              width: barWidth,
              borderRadius: BorderRadius.all(Radius.zero),
              borderSide: BorderSide(
                color: borderColor,
              ),
            ),
          ],
        ));
      }
    });

    return data;
  }

  FlBorderData getBorderData() {
    return FlBorderData(
      show: true,
      border: Border.all(
        color: AppColors.borderColor,
        width: 2,
      ),
    );
  }

  FlGridData getGridData() {
    return const FlGridData(
      show: true,
      drawHorizontalLine: true,
      drawVerticalLine: false,
    );
  }

  FlTitlesData getTitlesData() {
    const AxisTitles none = const AxisTitles(
      sideTitles: SideTitles(showTitles: false),
    );

    final AxisTitles verticalTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        reservedSize: 30,
        getTitlesWidget: getVerticalTitlesWidget,
        interval: 5,
      ),
    );
    final AxisTitles horizontalTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        reservedSize: 20,
        getTitlesWidget: getHorizontalTitlesWidget,
      ),
    );

    return FlTitlesData(
      show: true,
      bottomTitles: horizontalTitles,
      leftTitles: none,
      topTitles: none,
      rightTitles: verticalTitles,
    );
  }

  Widget getVerticalTitlesWidget(double value, TitleMeta meta) {
    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 4,
      child: Text(
        value.toInt().toString(),
        style: const TextStyle(
          color: AppColors.mainTextColor1,
          fontSize: 12,
        ),
      ),
    );
  }

  Widget getHorizontalTitlesWidget(double value, TitleMeta meta) {
    String text = '';

    if (value % 4 == 0 || value == 23) {
      text = value.toInt().toString().padLeft(2, '0');
    }

    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 2,
      child: Text(
        text,
        style: const TextStyle(
          color: AppColors.mainTextColor1,
          fontSize: 11,
        ),
      ),
    );
  }

  BarTouchData getBarTouchData() {
    return BarTouchData(
      enabled: true,
      touchTooltipData: BarTouchTooltipData(
        tooltipBgColor: Colors.transparent,
        tooltipPadding: EdgeInsets.zero,
        tooltipMargin: 2,
        getTooltipItem: (
          BarChartGroupData group,
          int groupIndex,
          BarChartRodData rod,
          int rodIndex,
        ) {
          return BarTooltipItem(
            rod.toY.round().toString(),
            const TextStyle(
              color: AppColors.mainTextColor2,
              fontWeight: FontWeight.bold,
              fontSize: 10,
            ),
          );
        },
      ),
    );
  }
}
